//! Contains the generated Chronicles of Darkness-related protocol
//! buffer types, as well as utilities and extensions for working with
//! them.

use std::collections::BTreeMap;

//Add the generated protobuf code into this module.
include!(concat!(env!("OUT_DIR"), "/models.proto.cofd.rs"));

//Add the API protobuf genreated code for the api module.
pub mod api {
    include!(concat!(env!("OUT_DIR"), "/models.proto.cofd.api.rs"));
}

const MENTAL_SKILLS: &'static [&'static str] = &[
    "Academics",
    "Computer",
    "Crafts",
    "Investigation",
    "Medicine",
    "Occult",
    "Politics",
    "Science",
];

const PHYSICAL_SKILLS: &'static [&'static str] = &[
    "Athletics",
    "Brawl",
    "Drive",
    "Firearms",
    "Larceny",
    "Stealth",
    "Survival",
    "Weaponry",
];

const SOCIAL_SKILLS: &'static [&'static str] = &[
    "Animal Ken",
    "Empathy",
    "Expression",
    "Intimidation",
    "Persuasion",
    "Socialize",
    "Streetwise",
    "Subterfuge",
];

/// Create a pre-populated skill list based on skill names given to
/// the function. The list of skill names is turned into a sorted Map
/// of skill name to default Skill protobuf instances.
fn create_skill_list(skill_names: &[&str]) -> BTreeMap<String, cofd_sheet::Skill> {
    skill_names
        .into_iter()
        .map(|skill_name| (skill_name.to_string(), cofd_sheet::Skill::default()))
        .collect()
}

impl CofdSheet {
    /// Create the default (blank) character sheet for a core
    /// Chronicles of Darkness character. This fills in skills and
    /// other information that needs to be pre-populated.
    pub fn default_sheet() -> CofdSheet {
        let mut sheet = Self::default();
        sheet.mental_skills = create_skill_list(&MENTAL_SKILLS);
        sheet.physical_skills = create_skill_list(&PHYSICAL_SKILLS);
        sheet.social_skills = create_skill_list(&SOCIAL_SKILLS);
        sheet
    }
}

impl ChangelingSheet {
    /// Create the default (blank) character sheet for a Changeling
    /// character. This fills in skills and other information that
    /// needs to be pre-populated.
    pub fn default_sheet() -> ChangelingSheet {
        let mut sheet = Self::default();
        sheet.base = Some(CofdSheet::default_sheet());
        //TODO fill in changeling-specific stuff
        sheet
    }
}

// TODO these values are not available in tera templates, so how to
// handle?
pub(crate) trait DerivedStats {
    fn speed(&self) -> i32;
}

impl DerivedStats for CofdSheet {
    fn speed(&self) -> i32 {
        self.size + self.stamina
    }
}
